﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinGrid;

namespace UTCToLocal
{
    #region Form1 class
    public partial class Form1 : Form
    {
        private DataTable table = null;

        public Form1()
        {
            InitializeComponent();

            this.grid.Dock = DockStyle.Fill;
            this.grid.DisplayLayout.AutoFitStyle = AutoFitStyle.ResizeAllColumns;
            this.grid.InitializeLayout += new InitializeLayoutEventHandler(grid_InitializeLayout);
            this.grid.InitializeRow += new InitializeRowEventHandler(grid_InitializeRow);
            this.grid.DataSource = this.Data;
            this.grid.AfterExitEditMode += new EventHandler(grid_AfterExitEditMode);

        }

        void grid_InitializeRow(object sender, InitializeRowEventArgs e)
        {
            UltraGridCell cell = e.Row.Cells["Local"];
            this.UpdateUnboundColumn( cell );
        }

        void grid_AfterExitEditMode(object sender, EventArgs e)
        {
            //  When cells in the "Local" column exit edit mode,
            //  assign the string representation of the cell's
            //  value to the "UTC" column's cell, just to demonstrate
            //  that the "Local" cell's underlying value is actually
            //  expressed as universal time.
            UltraGrid grid = sender as UltraGrid;
            UltraGridCell cell = grid.ActiveCell;
            this.UpdateUnboundColumn( cell );
        }

        private void UpdateUnboundColumn( UltraGridCell cell )
        {
            if ( cell.Column.Key == "Local" && cell.Value is DateTime )
            {
                UltraGridRow row = cell.Row;
                row.Cells["UTC"].Value = ((DateTime)cell.Value).ToString("g");
            }
        }

        void grid_InitializeLayout(object sender, InitializeLayoutEventArgs e)
        {
            ColumnsCollection columns = e.Layout.Bands[0].Columns;
            foreach ( UltraGridColumn column in columns )
            {
                string key = column.Key;

                switch ( key )
                {
                    case "TimeZone":
                        column.Editor = new TimeZoneEditor();
                        column.CellActivation = Activation.NoEdit;
                        break;

                    case "Local":
                        DateTimeEditor editor = new DateTimeEditor();
                        column.Format = "g";
                        column.MaskInput = string.Format( "{0} {1}", editor.CalcDefaultDateMask(null), editor.CalcDefaultTimeMask(null) );
                        editor.DataFilter = new UtcToLocalConversionDataFilter();
                        column.Editor = editor;
                        break;
                }

                //column.PerformAutoResize();
            }

            //  Add an unbound column to show the string representation
            //  of the universal date/time
            UltraGridColumn utcColumn = columns.Add( "UTC" );
            utcColumn.DataType = typeof(string);
            utcColumn.CellActivation = Activation.NoEdit;
        }

        private DataTable Data
        {
            get
            {
                if ( this.table == null )
                {
                    this.table = new DataTable();
                    this.table.Columns.Add( "TimeZone", typeof(string) );
                    this.table.Columns.Add( "Local", typeof(DateTime) );

                    //  Add a row for each time zone; we'll use the four continental
                    //  U.S. time zones for this demo.
                    this.table.Rows.Add( new object[]{ "Eastern Standard Time", DateTime.Today } );
                    this.table.Rows.Add( new object[]{ "Central Standard Time", DateTime.Today } );
                    this.table.Rows.Add( new object[]{ "Mountain Standard Time", DateTime.Today } );
                    this.table.Rows.Add( new object[]{ "Pacific Standard Time", DateTime.Today } );
                }

                return this.table;
            }
        }

    }
    #endregion Form1 class

    #region UtcToLocalConversionDataFilter class
    public class UtcToLocalConversionDataFilter : IEditorDataFilter
    {

        #region IEditorDataFilter Members

        object IEditorDataFilter.Convert(EditorDataFilterConvertArgs conversionArgs)
        {
            ConversionDirection direction = conversionArgs.Direction;
            UltraGridCell cell = conversionArgs.Context as UltraGridCell;
            DateTime value = conversionArgs.IsValid && conversionArgs.Value is DateTime ? (DateTime)conversionArgs.Value : DateTime.MinValue;

            if ( cell == null || value == DateTime.MinValue )
                return conversionArgs.Value;
            
            //  Get the TimeZoneEditor from the "TimeZone" column,
            //  and use it to get the TimeZoneInfo object from its
            //  value, which is the standard name of the time zone.
            TimeZoneEditor timeZoneEditor = cell.Row.Cells["TimeZone"].EditorResolved as TimeZoneEditor;
            string timeZoneName = cell.Row.Cells["TimeZone"].Value as string;
            Infragistics.Win.TimeZoneInfo tzi =
                string.IsNullOrEmpty(timeZoneName) == false ?
                Infragistics.Win.TimeZoneInfo.FromStandardName(timeZoneName) :
                null;

            if ( tzi == null )
                return conversionArgs.Value;

            switch ( direction )
            {
                case ConversionDirection.EditorToOwner:
                {
                    //  When the value is going from the editor back to
                    //  the cell, convert it to universal time.
                    
                    //  Note that for the current time zone, we can use the
                    //  DateTime.ToUniversalTime method, but that method
                    //  assumes that the specified time is local for the
                    //  current time zone, so we can't use it for each cell,
                    //  we have to "calculate" it using the TimeZoneInfo's
                    //  offsets.
                    DateTime universalTime = value;
                    universalTime = universalTime.Add( tzi.UtcOffset );
                    universalTime = universalTime.Add( tzi.DaylightUtcOffset );

                    conversionArgs.Handled = true;
                    return universalTime;
                }
                break;

                case ConversionDirection.OwnerToEditor:
                {
                    //  When the value is going from the cell to
                    //  the editor, convert it to local time.
                    DateTime localTime = tzi.ToLocalTime( value );
                    conversionArgs.Handled = true;
                    return localTime;
                }
                break;

            }

            return conversionArgs.Value;
        }

        #endregion
    }
    #endregion UtcToLocalConversionDataFilter class
}
